@@ -375,7 +375,7 @@ int DeStateDetectStartDetection(ThreadVars *tv, DetectEngineCtx *de_ctx,
 
     /* if continue detection already inspected this rule for this tx,
      * continue with the first not-inspected tx */
-    uint8_t offset = det_ctx->de_state_sig_array[s->num] & 0xef;
+    uint8_t offset = det_ctx->de_state_sig_array[s->num] & 0x7f;
     uint64_t tx_id = AppLayerParserGetTransactionInspectId(f->alparser, flags);
     if (offset > 0) {
         SCLogDebug("using stored_tx_id %"PRIu64" instead of %"PRIu64, tx_id+offset, tx_id);
