@@ -39,8 +39,10 @@ typedef struct OutputJSONMemBufferWrapper_ {
 
 int OutputJSONMemBufferCallback(const char *str, size_t size, void *data);
 
+void JsonAddVars(const Packet *p, const Flow *f, json_t *js);
 void CreateJSONFlowId(json_t *js, const Flow *f);
 void JsonTcpFlags(uint8_t flags, json_t *js);
+void JsonFiveTuple(const Packet *, int, json_t *);
 json_t *CreateJSONHeader(const Packet *p, int direction_sensative, const char *event_type);
 json_t *CreateJSONHeaderWithTxId(const Packet *p, int direction_sensitive, const char *event_type, uint64_t tx_id);
 int OutputJSONBuffer(json_t *js, LogFileCtx *file_ctx, MemBuffer **buffer);
@@ -62,6 +64,8 @@ typedef struct AlertJsonThread_ {
     LogFileCtx *file_ctx;
 } AlertJsonThread;
 
+json_t *SCJsonBool(int val);
+
 #endif /* HAVE_LIBJANSSON */
 
 #endif /* __OUTPUT_JSON_H__ */
