@@ -41,21 +41,7 @@ typedef enum {
     TMM_DECODEPCAPFILE,
     TMM_RECEIVEPFRING,
     TMM_DECODEPFRING,
-    TMM_ALERTFASTLOG,
-    TMM_ALERTUNIFIED2ALERT,
-    TMM_ALERTPRELUDE,
-    TMM_ALERTDEBUGLOG,
-    TMM_ALERTSYSLOG,
-    TMM_LOGDROPLOG,
     TMM_RESPONDREJECT,
-    TMM_LOGDNSLOG,
-    TMM_LOGHTTPLOG,
-    TMM_LOGTLSLOG,
-    TMM_LOGTCPDATALOG,
-    TMM_OUTPUTJSON,
-    TMM_PCAPLOG,
-    TMM_FILELOG,
-    TMM_FILESTORE,
     TMM_DECODEIPFW,
     TMM_VERDICTIPFW,
     TMM_RECEIVEIPFW,
@@ -72,27 +58,9 @@ typedef enum {
     TMM_DECODEMPIPE,
     TMM_RECEIVENAPATECH,
     TMM_DECODENAPATECH,
-    TMM_PACKETLOGGER,
-    TMM_TXLOGGER,
     TMM_STATSLOGGER,
-    TMM_FILELOGGER,
-    TMM_FILEDATALOGGER,
-    TMM_STREAMINGLOGGER,
-    TMM_JSONALERTLOG,
-    TMM_JSONDROPLOG,
-    TMM_JSONHTTPLOG,
-    TMM_JSONDNSLOG,
-    TMM_JSONSMTPLOG,
-    TMM_JSONSSHLOG,
-    TMM_JSONSTATSLOG,
-    TMM_JSONTLSLOG,
-    TMM_JSONFILELOG,
     TMM_RECEIVENFLOG,
     TMM_DECODENFLOG,
-    TMM_JSONFLOWLOG,
-    TMM_JSONNETFLOWLOG,
-    TMM_LOGSTATSLOG,
-    TMM_JSONTEMPLATELOG,
 
     TMM_FLOWMANAGER,
     TMM_FLOWRECYCLER,
@@ -100,8 +68,6 @@ typedef enum {
 
     TMM_UNIXMANAGER,
 
-    TMM_LUALOG,
-    TMM_TLSSTORE,
     TMM_SIZE,
 } TmmId;
 
